@@ -1,48 +1,55 @@
 /*
- *  Licensed to the Apache Software Foundation (ASF) under one
- *  or more contributor license agreements.  See the NOTICE file
- *  distributed with this work for additional information
- *  regarding copyright ownership.  The ASF licenses this file
- *  to you under the Apache License, Version 2.0 (the
- *  "License"); you may not use this file except in compliance
- *  with the License.  You may obtain a copy of the License at
- * 
- *  http://www.apache.org/licenses/LICENSE-2.0
- * 
- *  Unless required by applicable law or agreed to in writing,
- *  software distributed under the License is distributed on an
- *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
- *  KIND, either express or implied.  See the License for the
- *  specific language governing permissions and limitations
- *  under the License.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 package org.apache.myfaces.shared_tomahawk.util;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
-
+import javax.el.ExpressionFactory;
 import javax.faces.FacesException;
 import javax.faces.context.FacesContext;
-import javax.el.ExpressionFactory;
-import java.io.InputStream;
 import java.io.IOException;
+import java.io.InputStream;
 import java.lang.reflect.Array;
-import java.security.AccessController;
-import java.security.PrivilegedActionException;
-import java.security.PrivilegedExceptionAction;
-import java.util.*;
+import java.lang.reflect.Constructor;
+import java.lang.reflect.InvocationTargetException;
+import java.net.URL;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
+import java.util.logging.Level;
+import java.util.logging.Logger;
 
 
 /**
- * @author Manfred Geiler (latest modification by $Author: werpu $)
+ * @author Manfred Geiler (latest modification by $Author: bommel $)
  * @author Anton Koinov
- * @version $Revision: 805066 $ $Date: 2009-08-17 12:39:52 -0500 (Mon, 17 Aug 2009) $
+ * @version $Revision: 1187700 $ $Date: 2011-10-22 07:19:37 -0500 (Sat, 22 Oct 2011) $
  */
 public final class ClassUtils
 {
     //~ Static fields/initializers -----------------------------------------------------------------
 
-    private static final Log log                  = LogFactory.getLog(ClassUtils.class);
+    //private static final Log log                  = LogFactory.getLog(ClassUtils.class);
+    private static final Logger log                  = Logger.getLogger(ClassUtils.class.getName());
 
     public static final Class BOOLEAN_ARRAY_CLASS = boolean[].class;
     public static final Class BYTE_ARRAY_CLASS    = byte[].class;
@@ -119,18 +126,26 @@ public final class ClassUtils
 
     //~ Methods ------------------------------------------------------------------------------------
 
-    public static void addClassLoadingExtension(ClassLoaderExtension extension, boolean top) {
-        /**
-               * now at the first look this looks somewhat strange
-               * but to avoid synchronzed access we assign new native
-               * arrays to our static variable
-               * since primitive assignments are atomic in java and
-               * an array  falls under the category primitive I assume we can
-               * work the lisp way of immutable data structures
-               */
+    public synchronized static void addClassLoadingExtension(ClassLoaderExtension extension, boolean top) {
+      /**
+       * now at the first look this looks somewhat strange
+       * but to get the best performance access we assign new native
+       * arrays to our static variable
+       * 
+       * we have to synchronized nevertheless because if two threads try to register
+       * loaders at the same time none of them should get lost
+       */
         ClassLoaderExtension [] retVal = new ClassLoaderExtension[classLoadingExtensions.length+1];
         ArrayList extensions = new ArrayList(classLoadingExtensions.length+1);
+
+        if(!top) {
+            extensions.addAll(Arrays.asList(classLoadingExtensions));
+        }
         extensions.add(extension);
+        if(top) {
+            extensions.addAll(Arrays.asList(classLoadingExtensions));
+        }    
+
         classLoadingExtensions = (ClassLoaderExtension []) extensions.toArray(retVal);
     }
 
@@ -196,7 +211,7 @@ public final class ClassUtils
         }
         catch (ClassNotFoundException e)
         {
-            log.error("Class " + type + " not found", e);
+            log.log(Level.SEVERE, "Class " + type + " not found", e);
             throw new FacesException(e);
         }
     }
@@ -253,11 +268,21 @@ public final class ClassUtils
         }
         catch (ClassNotFoundException e)
         {
-            log.error("Class " + type + " not found", e);
+            log.log(Level.SEVERE, "Class " + type + " not found", e);
             throw new FacesException(e);
         }
     }
 
+    public static URL getResource(String resource)
+    {
+        URL url = getContextClassLoader().getResource(resource);
+        if (url == null)
+        {
+            url = ClassUtils.class.getClassLoader().getResource(resource);
+        }
+        return url;
+    }
+
     public static InputStream getResourceAsStream(String resource)
     {
         InputStream stream = getContextClassLoader()
@@ -289,7 +314,7 @@ public final class ClassUtils
         }
         catch (IOException e)
         {
-            log.error(e.getMessage(), e);
+            log.log(Level.SEVERE, e.getMessage(), e);
             throw new FacesException(e);
         }
     }
@@ -329,7 +354,7 @@ public final class ClassUtils
         return newInstance(clazzForName);
     }
 
-    public static Object newInstance(Class clazz)
+    public static <T> T newInstance(Class<T> clazz)
         throws FacesException
     {
         try
@@ -338,17 +363,41 @@ public final class ClassUtils
         }
         catch(NoClassDefFoundError e)
         {
-            log.error("Class : "+clazz.getName()+" not found.",e);
+            log.log(Level.SEVERE, "Class : "+clazz.getName()+" not found.",e);
             throw new FacesException(e);
         }
         catch (InstantiationException e)
         {
-            log.error(e.getMessage(), e);
+            log.log(Level.SEVERE, e.getMessage(), e);
             throw new FacesException(e);
         }
         catch (IllegalAccessException e)
         {
-            log.error(e.getMessage(), e);
+            log.log(Level.SEVERE, e.getMessage(), e);
+            throw new FacesException(e);
+        }
+    }
+
+    public static <T> T newInstance(Class<T> clazz,
+                                    Class<?>[] constructorArgClasses,
+                                    Object... constructorArgs) throws NoSuchMethodException
+    {
+        if (constructorArgs.length == 0)
+        {
+            // no args given - use normal newInstance()
+            return newInstance(clazz);
+        }
+
+        // try to get a fitting constructor (throws NoSuchMethodException)
+        Constructor constructor = clazz.getConstructor(constructorArgClasses);
+
+        try
+        {
+            // actually create instance
+            return (T) constructor.newInstance(constructorArgs);
+        }
+        catch (Exception e)
+        {
             throw new FacesException(e);
         }
     }
@@ -366,7 +415,7 @@ public final class ClassUtils
         {
             String message = "Cannot coerce " + value.getClass().getName()
                              + " to " + desiredClass.getName();
-            log.error(message, e);
+            log.log(Level.SEVERE, message, e);
             throw new FacesException(message, e);
         }
     }
@@ -397,26 +446,159 @@ public final class ClassUtils
      */
     public static ClassLoader getContextClassLoader()
     {
-        if (System.getSecurityManager() != null) 
+        // call into the same method on ClassLoaderUtils.  no need for duplicate code maintenance. 
+        return ClassLoaderUtils.getContextClassLoader();
+    }
+    
+    /**
+     * Creates ApplicationObjects like NavigationHandler or StateManager and creates 
+     * the right wrapping chain of the ApplicationObjects known as the decorator pattern. 
+     * @param <T>
+     * @param interfaceClass The class from which the implementation has to inherit from.
+     * @param classNamesIterator All the class names of the actual ApplicationObject implementations
+     *                           from the faces-config.xml.
+     * @param defaultObject The default implementation for the given ApplicationObject.
+     * @return
+     */    
+    public static <T> T buildApplicationObject(Class<T> interfaceClass, Collection<String> classNamesIterator, T defaultObject)
+    {
+        return buildApplicationObject(interfaceClass, null, null, classNamesIterator, defaultObject);
+    }
+
+    /**
+     * Creates ApplicationObjects like NavigationHandler or StateManager and creates 
+     * the right wrapping chain of the ApplicationObjects known as the decorator pattern. 
+     * @param <T>
+     * @param interfaceClass The class from which the implementation has to inherit from.
+     * @param extendedInterfaceClass A subclass of interfaceClass which specifies a more
+     *                               detailed implementation.
+     * @param extendedInterfaceWrapperClass A wrapper class for the case that you have an ApplicationObject
+     *                                      which only implements the interfaceClass but not the 
+     *                                      extendedInterfaceClass.
+     * @param classNamesIterator All the class names of the actual ApplicationObject implementations
+     *                           from the faces-config.xml.
+     * @param defaultObject The default implementation for the given ApplicationObject.
+     * @return
+     */
+    @SuppressWarnings("unchecked")
+    public static <T> T buildApplicationObject(Class<T> interfaceClass, Class<? extends T> extendedInterfaceClass,
+            Class<? extends T> extendedInterfaceWrapperClass,
+            Collection<String> classNamesIterator, T defaultObject)
+    {
+        T current = defaultObject;
+        
+
+        for (String implClassName : classNamesIterator)
         {
-            try {
-                ClassLoader cl = AccessController.doPrivileged(new PrivilegedExceptionAction<ClassLoader>()
-                        {
-                            public ClassLoader run() throws PrivilegedActionException
-                            {
-                                return Thread.currentThread().getContextClassLoader();
-                            }
-                        });
-                return cl;
-            }
-            catch (PrivilegedActionException pae)
+            Class<? extends T> implClass = ClassUtils.simpleClassForName(implClassName);
+
+            // check, if class is of expected interface type
+            if (!interfaceClass.isAssignableFrom(implClass))
             {
-                throw new FacesException(pae);
+                throw new IllegalArgumentException("Class " + implClassName + " is no " + interfaceClass.getName());
+            }
+
+            if (current == null)
+            {
+                // nothing to decorate
+                current = (T) ClassUtils.newInstance(implClass);
+            }
+            else
+            {
+                // let's check if class supports the decorator pattern
+                T newCurrent = null;
+                try
+                {
+                    Constructor<? extends T> delegationConstructor = null;
+                    
+                    // first, if there is a extendedInterfaceClass,
+                    // try to find a constructor that uses that
+                    if (extendedInterfaceClass != null 
+                            && extendedInterfaceClass.isAssignableFrom(current.getClass()))
+                    {
+                        try
+                        {
+                            delegationConstructor = 
+                                    implClass.getConstructor(new Class[] {extendedInterfaceClass});
+                        }
+                        catch (NoSuchMethodException mnfe)
+                        {
+                            // just eat it
+                        }
+                    }
+                    if (delegationConstructor == null)
+                    {
+                        // try to find the constructor with the "normal" interfaceClass
+                        delegationConstructor = 
+                                implClass.getConstructor(new Class[] {interfaceClass});
+                    }
+                    // impl class supports decorator pattern at this point
+                    try
+                    {
+                        // create new decorator wrapping current
+                        newCurrent = delegationConstructor.newInstance(new Object[] { current });
+                    }
+                    catch (InstantiationException e)
+                    {
+                        log.log(Level.SEVERE, e.getMessage(), e);
+                        throw new FacesException(e);
+                    }
+                    catch (IllegalAccessException e)
+                    {
+                        log.log(Level.SEVERE, e.getMessage(), e);
+                        throw new FacesException(e);
+                    }
+                    catch (InvocationTargetException e)
+                    {
+                        log.log(Level.SEVERE, e.getMessage(), e);
+                        throw new FacesException(e);
+                    }
+                }
+                catch (NoSuchMethodException e)
+                {
+                    // no decorator pattern support
+                    newCurrent = (T) ClassUtils.newInstance(implClass);
+                }
+                
+                // now we have a new current object (newCurrent)
+                // --> find out if it is assignable from extendedInterfaceClass
+                // and if not, wrap it in a backwards compatible wrapper (if available)
+                if (extendedInterfaceWrapperClass != null
+                        && !extendedInterfaceClass.isAssignableFrom(newCurrent.getClass()))
+                {
+                    try
+                    {
+                        Constructor<? extends T> wrapperConstructor
+                                = extendedInterfaceWrapperClass.getConstructor(
+                                        new Class[] {interfaceClass, extendedInterfaceClass});
+                        newCurrent = wrapperConstructor.newInstance(new Object[] {newCurrent, current});
+                    }
+                    catch (NoSuchMethodException e)
+                    {
+                        log.log(Level.SEVERE, e.getMessage(), e);
+                        throw new FacesException(e);
+                    }
+                    catch (InstantiationException e)
+                    {
+                        log.log(Level.SEVERE, e.getMessage(), e);
+                        throw new FacesException(e);
+                    }
+                    catch (IllegalAccessException e)
+                    {
+                        log.log(Level.SEVERE, e.getMessage(), e);
+                        throw new FacesException(e);
+                    }
+                    catch (InvocationTargetException e)
+                    {
+                        log.log(Level.SEVERE, e.getMessage(), e);
+                        throw new FacesException(e);
+                    }
+                }
+                
+                current = newCurrent;
             }
         }
-        else
-        {
-            return Thread.currentThread().getContextClassLoader();
-        }
+
+        return current;
     }
 }
