@@ -1,20 +1,20 @@
 /*
- *  Licensed to the Apache Software Foundation (ASF) under one
- *  or more contributor license agreements.  See the NOTICE file
- *  distributed with this work for additional information
- *  regarding copyright ownership.  The ASF licenses this file
- *  to you under the Apache License, Version 2.0 (the
- *  "License"); you may not use this file except in compliance
- *  with the License.  You may obtain a copy of the License at
- * 
- *  http://www.apache.org/licenses/LICENSE-2.0
- * 
- *  Unless required by applicable law or agreed to in writing,
- *  software distributed under the License is distributed on an
- *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
- *  KIND, either express or implied.  See the License for the
- *  specific language governing permissions and limitations
- *  under the License.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 package org.apache.myfaces.shared_tomahawk.util;
 
@@ -22,6 +22,8 @@ import java.text.MessageFormat;
 import java.util.Locale;
 import java.util.MissingResourceException;
 import java.util.ResourceBundle;
+import java.util.logging.Level;
+import java.util.logging.Logger;
 
 import javax.el.ValueExpression;
 import javax.faces.FactoryFinder;
@@ -31,9 +33,6 @@ import javax.faces.application.FacesMessage;
 import javax.faces.component.UIComponent;
 import javax.faces.context.FacesContext;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
-
 /**
  * Utility class to support multilingual FacesMessages using ResourceBundles.
  * Standard messages are stored at <code>DEFAULT_BUNDLE</code>.<br>
@@ -47,7 +46,7 @@ import org.apache.commons.logging.LogFactory;
  * @author Manfred Geiler
  * @author Sean Schofield
  * @author Stpehan Strittmatter
- * @version $Revision: 940133 $ $Date: 2010-05-01 20:30:31 -0500 (Sat, 01 May 2010) $
+ * @version $Revision: 1380845 $ $Date: 2012-09-04 15:10:04 -0500 (Tue, 04 Sep 2012) $
  */
 public final class MessageUtils
 {
@@ -62,7 +61,8 @@ public final class MessageUtils
 
     /** Suffix for message details (<code>_detail</code>)*/
     private static final String DETAIL_SUFFIX = "_detail";
-    private static Log log = LogFactory.getLog(MessageUtils.class);
+    //private static Log log = LogFactory.getLog(MessageUtils.class);
+    private static Logger log = Logger.getLogger(MessageUtils.class.getName());
 
     /**
      * @param severity serverity of message
@@ -200,8 +200,9 @@ public final class MessageUtils
                                   String forClientId,
                                   FacesContext facesContext)
     {
-        if(log.isTraceEnabled()) {
-          log.trace("adding message " + messageId + " for clientId " + forClientId);
+        if(log.isLoggable(Level.FINEST))
+        {
+          log.finest("adding message " + messageId + " for clientId " + forClientId);
         }
         facesContext.addMessage(forClientId,
                                 getMessage(severity, messageId, args, facesContext));
@@ -214,8 +215,9 @@ public final class MessageUtils
             String forClientId,
             FacesContext facesContext)
     {
-        if(log.isTraceEnabled()) {
-        log.trace("adding message " + messageId + " for clientId " + forClientId);
+        if(log.isLoggable(Level.FINEST))
+        {
+          log.finest("adding message " + messageId + " for clientId " + forClientId);
         }
         facesContext.addMessage(forClientId,
                   getMessage(bundleBaseName, severity, messageId, args, facesContext));
@@ -233,7 +235,9 @@ public final class MessageUtils
     {
         String localizedStr = null;
         if(params == null || msgtext == null)
+        {
             return msgtext;
+        }
 
         if(locale != null)
         {
@@ -257,8 +261,11 @@ public final class MessageUtils
         {
             locale = context.getViewRoot().getLocale();
             if(locale == null)
+            {
                 locale = Locale.getDefault();
-        } else
+            }
+        }
+        else
         {
             locale = Locale.getDefault();
         }
@@ -276,10 +283,12 @@ public final class MessageUtils
         {
             try
             {
-                bundle = ResourceBundle.getBundle(bundleName, locale, org.apache.myfaces.shared_tomahawk.util.ClassUtils.getCurrentLoader(bundleName));
+                bundle = ResourceBundle.getBundle(bundleName, locale, 
+                        org.apache.myfaces.shared_tomahawk.util.ClassUtils.getCurrentLoader(bundleName));
                 summary = bundle.getString(messageId);
             }
-            catch (MissingResourceException e) {
+            catch (MissingResourceException e)
+            {
                 // NoOp
             }
         }
@@ -288,14 +297,16 @@ public final class MessageUtils
         {
             try
             {
-                bundle = ResourceBundle.getBundle(DEFAULT_BUNDLE, locale, org.apache.myfaces.shared_tomahawk.util.ClassUtils.getCurrentLoader(DEFAULT_BUNDLE));
+                bundle = ResourceBundle.getBundle(DEFAULT_BUNDLE, locale, 
+                        org.apache.myfaces.shared_tomahawk.util.ClassUtils.getCurrentLoader(DEFAULT_BUNDLE));
                 if(bundle == null)
                 {
                     throw new NullPointerException();
                 }
                 summary = bundle.getString(messageId);
             }
-            catch(MissingResourceException e) {
+            catch(MissingResourceException e)
+            {
                 // NoOp
             }
         }
@@ -316,7 +327,8 @@ public final class MessageUtils
             {
                 detail = bundle.getString(messageId + DETAIL_SUFFIX);
             }
-            catch(MissingResourceException e) {
+            catch(MissingResourceException e)
+            {
                 // NoOp
             }
             return new ParametrizableFacesMessage(summary, detail, params, locale);
@@ -329,14 +341,16 @@ public final class MessageUtils
                 detail = substituteParams(locale,
                     bundle.getString(messageId + DETAIL_SUFFIX), params);
             }
-            catch(MissingResourceException e) {
+            catch(MissingResourceException e)
+            {
                 // NoOp
             }
             return new FacesMessage(summary, detail);
         }
     }
     
-    public static FacesMessage getMessageFromBundle(String bundleBaseName, FacesContext context, Locale locale, String messageId, Object params[])
+    public static FacesMessage getMessageFromBundle(String bundleBaseName, 
+            FacesContext context, Locale locale, String messageId, Object params[])
     {
         String summary = null;
         String detail = null;
@@ -347,10 +361,12 @@ public final class MessageUtils
         {
             try
             {
-                bundle = ResourceBundle.getBundle(bundleName, locale, org.apache.myfaces.shared_tomahawk.util.ClassUtils.getCurrentLoader(bundleName));
+                bundle = ResourceBundle.getBundle(bundleName, locale, 
+                        org.apache.myfaces.shared_tomahawk.util.ClassUtils.getCurrentLoader(bundleName));
                 summary = bundle.getString(messageId);
             }
-            catch (MissingResourceException e) {
+            catch (MissingResourceException e)
+            {
                 // NoOp
             }
         }
@@ -359,14 +375,16 @@ public final class MessageUtils
         {
             try
             {
-                bundle = ResourceBundle.getBundle(bundleBaseName, locale, org.apache.myfaces.shared_tomahawk.util.ClassUtils.getCurrentLoader(bundleBaseName));
+                bundle = ResourceBundle.getBundle(bundleBaseName, locale, 
+                        org.apache.myfaces.shared_tomahawk.util.ClassUtils.getCurrentLoader(bundleBaseName));
                 if(bundle == null)
                 {
                     throw new NullPointerException();
                 }
                 summary = bundle.getString(messageId);
             }
-            catch(MissingResourceException e) {
+            catch(MissingResourceException e)
+            {
                 // NoOp
             }
         }
@@ -375,14 +393,16 @@ public final class MessageUtils
         {
             try
             {
-                bundle = ResourceBundle.getBundle(DEFAULT_BUNDLE, locale, org.apache.myfaces.shared_tomahawk.util.ClassUtils.getCurrentLoader(DEFAULT_BUNDLE));
+                bundle = ResourceBundle.getBundle(DEFAULT_BUNDLE, locale, 
+                        org.apache.myfaces.shared_tomahawk.util.ClassUtils.getCurrentLoader(DEFAULT_BUNDLE));
                 if(bundle == null)
                 {
                     throw new NullPointerException();
                 }
                 summary = bundle.getString(messageId);
             }
-            catch(MissingResourceException e) {
+            catch(MissingResourceException e)
+            {
                 // NoOp
             }
         }
@@ -404,7 +424,8 @@ public final class MessageUtils
             {
                 detail = bundle.getString(messageId + DETAIL_SUFFIX);
             }
-            catch(MissingResourceException e) {
+            catch(MissingResourceException e)
+            {
                 // NoOp
             }
             return new ParametrizableFacesMessage(summary, detail, params, locale);
@@ -417,7 +438,8 @@ public final class MessageUtils
                 detail = substituteParams(locale,
                     bundle.getString(messageId + DETAIL_SUFFIX), params);
             }
-            catch(MissingResourceException e) {
+            catch(MissingResourceException e)
+            {
                 // NoOp
             }
             return new FacesMessage(summary, detail);
@@ -442,17 +464,21 @@ public final class MessageUtils
      * 
      * @return  currently applicable Locale for this request.
      */
-    public static Locale getCurrentLocale() {
+    public static Locale getCurrentLocale()
+    {
         return getCurrentLocale(FacesContext.getCurrentInstance());
     }
     
-    public static Locale getCurrentLocale(FacesContext context) {
+    public static Locale getCurrentLocale(FacesContext context)
+    {
         Locale locale;
         if(context != null && context.getViewRoot() != null)
         {
             locale = context.getViewRoot().getLocale();
             if(locale == null)
+            {
                 locale = Locale.getDefault();
+            }
         }
         else
         {
@@ -469,7 +495,8 @@ public final class MessageUtils
      * @param params parameters to set at localized message
      * @return generated FacesMessage
      */
-    public static FacesMessage getMessage(FacesMessage.Severity severity, String bundleBaseName, String messageId, Object params[])
+    public static FacesMessage getMessage(FacesMessage.Severity severity, String bundleBaseName, 
+            String messageId, Object params[])
     {
       FacesMessage msg = getMessage(bundleBaseName, messageId, params);
       msg.setSeverity(severity);
@@ -515,7 +542,8 @@ public final class MessageUtils
       {
           summary = bundle.getString(messageId);
       }
-      catch (MissingResourceException e) {
+      catch (MissingResourceException e)
+      {
         // NoOp
       }
 
@@ -532,7 +560,8 @@ public final class MessageUtils
           detail = substituteParams(bundle.getLocale(),
               bundle.getString(messageId + DETAIL_SUFFIX), params);
       }
-      catch(MissingResourceException e) {
+      catch(MissingResourceException e)
+      {
         // NoOp
       }
 
@@ -565,15 +594,20 @@ public final class MessageUtils
     public static FacesMessage getMessage(FacesContext context, String messageId, Object params[])
     {
         if(context == null || messageId == null)
+        {
             throw new NullPointerException(" context " + context + " messageId " + messageId);
+        }
         Locale locale = getCurrentLocale(context);
         if(null == locale)
+        {
             throw new NullPointerException(" locale " + locale);
+        }
         FacesMessage message = getMessage(locale, messageId, params);
         if(message != null)
         {
             return message;
-        } else
+        } 
+        else
         {
             // TODO /FIX:  Note that this has fallback behavior to default Locale for message,
             // but similar behavior above does not.  The methods should probably behave
@@ -582,18 +616,24 @@ public final class MessageUtils
         }
     }
     
-    public static FacesMessage getMessage(String bundleBaseName, FacesContext context, String messageId, Object params[])
+    public static FacesMessage getMessage(String bundleBaseName, FacesContext context, 
+            String messageId, Object params[])
     {
         if(context == null || messageId == null)
+        {
             throw new NullPointerException(" context " + context + " messageId " + messageId);
+        }
         Locale locale = getCurrentLocale(context);
         if(null == locale)
+        {
             throw new NullPointerException(" locale " + locale);
+        }
         FacesMessage message = getMessageFromBundle(bundleBaseName, context, locale, messageId, params);
         if(message != null)
         {
             return message;
-        } else
+        } 
+        else
         {
             // TODO /FIX:  Note that this has fallback behavior to default Locale for message,
             // but similar behavior above does not.  The methods should probably behave
@@ -602,14 +642,40 @@ public final class MessageUtils
         }
     }
     
-    public static Object getLabel(FacesContext facesContext, UIComponent component) {
+    public static Object getLabel(FacesContext facesContext, UIComponent component)
+    {
         Object label = component.getAttributes().get("label");
+        ValueExpression expression = null;
+        if (label != null && 
+            label instanceof String && ((String)label).length() == 0 )
+        {
+            // Note component.getAttributes().get("label") internally try to 
+            // evaluate the EL expression for the label, but in some cases, 
+            // when PSS is disabled and f:loadBundle is used, when the view is 
+            // restored the bundle is not set to the EL expression returns an 
+            // empty String. It is not possible to check if there is a 
+            // hardcoded label, but we can check if there is
+            // an EL expression set, so the best in this case is use that, and if
+            // there is an EL expression set, use it, otherwise use the hardcoded
+            // value. See MYFACES-3591 for details.
+            expression = component.getValueExpression("label");
+            if (expression != null)
+            {
+                // Set the label to null and use the EL expression instead.
+                label = null;
+            }
+        }
+            
         if(label != null)
+        {
             return label;
+        }
         
-        ValueExpression expression = component.getValueExpression("label");
+        expression = (expression == null) ? component.getValueExpression("label") : expression;
         if(expression != null)
+        {
             return expression;
+        }
         
         //If no label is not specified, use clientId
         return component.getClientId( facesContext );
@@ -621,9 +687,11 @@ public final class MessageUtils
         if(context != null)
         {
             return context.getApplication();
-        } else
+        }
+        else
         {
-            ApplicationFactory afactory = (ApplicationFactory)FactoryFinder.getFactory("javax.faces.application.ApplicationFactory");
+            ApplicationFactory afactory = (ApplicationFactory)FactoryFinder.getFactory(
+                    "javax.faces.application.ApplicationFactory");
             return afactory.getApplication();
         }
     }
